@@ -1,16 +1,16 @@
 //! The "parse-tree" is what is produced by the parser. We use it do
 //! some pre-expansion and so forth before creating the proper AST.
 
-use grammar::consts::{INPUT_LIFETIME, LALR, RECURSIVE_ASCENT, TABLE_DRIVEN, TEST_ALL};
-use grammar::pattern::Pattern;
-use grammar::repr::{self as r, NominalTypeRepr, TypeRepr};
-use lexer::dfa::DFA;
-use message::builder::InlineBuilder;
-use message::Content;
+use crate::grammar::consts::{INPUT_LIFETIME, LALR, RECURSIVE_ASCENT, TABLE_DRIVEN, TEST_ALL};
+use crate::grammar::pattern::Pattern;
+use crate::grammar::repr::{self as r, NominalTypeRepr, TypeRepr};
+use crate::lexer::dfa::DFA;
+use crate::message::builder::InlineBuilder;
+use crate::message::Content;
+use crate::tls::Tls;
+use crate::util::Sep;
 use std::fmt::{Debug, Display, Error, Formatter};
 use string_cache::DefaultAtom as Atom;
-use tls::Tls;
-use util::Sep;
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Grammar {
@@ -25,7 +25,7 @@ pub struct Grammar {
     pub module_attributes: Vec<String>,
 }
 
-#[derive(Copy, Clone, Debug, Hash, PartialEq, Eq, PartialOrd, Ord)]
+#[derive(Copy, Clone, Debug, Default, Hash, PartialEq, Eq, PartialOrd, Ord)]
 pub struct Span(pub usize, pub usize);
 
 impl Into<Box<dyn Content>> for Span {
@@ -109,7 +109,29 @@ impl MatchItem {
 }
 
 pub type MatchSymbol = TerminalLiteral;
-pub type MatchMapping = TerminalString;
+
+#[derive(Clone, PartialEq, Eq, Ord, PartialOrd)]
+pub enum MatchMapping {
+    Terminal(TerminalString),
+    Skip,
+}
+
+impl Debug for MatchMapping {
+    fn fmt(&self, fmt: &mut Formatter) -> Result<(), Error> {
+        match self {
+            MatchMapping::Terminal(term) => write!(fmt, "{:?}", term),
+            MatchMapping::Skip => write!(fmt, "{{ }}"),
+        }
+    }
+}
+impl Display for MatchMapping {
+    fn fmt(&self, fmt: &mut Formatter) -> Result<(), Error> {
+        match self {
+            MatchMapping::Terminal(term) => write!(fmt, "{}", term),
+            MatchMapping::Skip => write!(fmt, "{{ }}"),
+        }
+    }
+}
 
 /// Intern tokens are not typed by the user: they are synthesized in
 /// the absence of an "extern" declaration with information about the
@@ -158,7 +180,7 @@ pub struct MatchEntry {
     /// NB: This field must go first, so that `PartialOrd` sorts by precedence first!
     pub precedence: usize,
     pub match_literal: TerminalLiteral,
-    pub user_name: TerminalString,
+    pub user_name: MatchMapping,
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -200,6 +222,9 @@ pub enum TypeRef {
     // (T1, T2)
     Tuple(Vec<TypeRef>),
 
+    // [T]
+    Slice(Box<TypeRef>),
+
     // Foo<'a, 'b, T1, T2>, Foo::Bar, etc
     Nominal {
         path: Path,
@@ -343,6 +368,7 @@ pub struct Parameter {
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum Visibility {
     Pub(Option<Path>),
+    PubIn(Path),
     Priv,
 }
 
@@ -350,6 +376,7 @@ impl Visibility {
     pub fn is_pub(&self) -> bool {
         match *self {
             Visibility::Pub(_) => true,
+            Visibility::PubIn(_) => true,
             Visibility::Priv => false,
         }
     }
@@ -384,6 +411,8 @@ pub struct Alternative {
 
     // => { code }
     pub action: Option<ActionKind>,
+
+    pub annotations: Vec<Annotation>,
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -713,6 +742,7 @@ impl Display for Visibility {
         match *self {
             Visibility::Pub(Some(ref path)) => write!(fmt, "pub({}) ", path),
             Visibility::Pub(None) => write!(fmt, "pub "),
+            Visibility::PubIn(ref path) => write!(fmt, "pub(in {}) ", path),
             Visibility::Priv => Ok(()),
         }
     }
@@ -1008,6 +1038,7 @@ impl Display for TypeRef {
     fn fmt(&self, fmt: &mut Formatter) -> Result<(), Error> {
         match *self {
             TypeRef::Tuple(ref types) => write!(fmt, "({})", Sep(", ", types)),
+            TypeRef::Slice(ref ty) => write!(fmt, "[{}]", ty),
             TypeRef::Nominal {
                 ref path,
                 ref types,
@@ -1076,6 +1107,7 @@ impl TypeRef {
             TypeRef::Tuple(ref types) => {
                 TypeRepr::Tuple(types.iter().map(TypeRef::type_repr).collect())
             }
+            TypeRef::Slice(ref ty) => TypeRepr::Slice(Box::new(ty.type_repr())),
             TypeRef::Nominal {
                 ref path,
                 ref types,
@@ -1144,16 +1176,16 @@ impl Path {
 
     pub fn vec() -> Path {
         Path {
-            absolute: true,
-            ids: vec![Atom::from("std"), Atom::from("vec"), Atom::from("Vec")],
+            absolute: false,
+            ids: vec![Atom::from("alloc"), Atom::from("vec"), Atom::from("Vec")],
         }
     }
 
     pub fn option() -> Path {
         Path {
-            absolute: true,
+            absolute: false,
             ids: vec![
-                Atom::from("std"),
+                Atom::from("core"),
                 Atom::from("option"),
                 Atom::from("Option"),
             ],
