@@ -2,10 +2,10 @@
 //! really of interest, we represent this just as a vector of labeled
 //! edges.
 
-use lexer::re::Regex;
+use crate::lexer::re::Regex;
 use regex_syntax::hir::{
-    Anchor, Class, ClassUnicodeRange, GroupKind, Hir, HirKind, Literal, RepetitionKind,
-    RepetitionRange,
+    Anchor, Class, ClassBytesRange, ClassUnicodeRange, GroupKind, Hir, HirKind, Literal,
+    RepetitionKind, RepetitionRange,
 };
 use std::char;
 use std::fmt::{Debug, Error as FmtError, Formatter};
@@ -212,8 +212,13 @@ impl NFA {
                         self.push_edge(s0, Other, reject);
                         Ok(s0)
                     }
-                    // Bytes are not supported
-                    Literal::Byte(_) => Err(NFAConstructionError::ByteRegex),
+                    //// Bytes are not supported
+                    Literal::Byte(b) => {
+                        let s0 = self.new_state(StateKind::Neither);
+                        self.push_edge(s0, Test::byte(b), accept);
+                        self.push_edge(s0, Other, reject);
+                        Ok(s0)
+                    }
                 }
             }
 
@@ -235,8 +240,16 @@ impl NFA {
                         self.push_edge(s0, Other, reject);
                         Ok(s0)
                     }
-                    // Bytes are not supported
-                    Class::Bytes(_) => Err(NFAConstructionError::ByteRegex),
+                    //// Bytes are not supported
+                    Class::Bytes(ref byte) => {
+                        let s0 = self.new_state(StateKind::Neither);
+                        for &range in byte.iter() {
+                            let test: Test = range.into();
+                            self.push_edge(s0, test, accept);
+                        }
+                        self.push_edge(s0, Other, reject);
+                        Ok(s0)
+                    }
                 }
             }
 
@@ -502,6 +515,14 @@ impl Test {
         }
     }
 
+    pub fn byte(b: u8) -> Test {
+        let b = b as u32;
+        Test {
+            start: b,
+            end: b + 1,
+        }
+    }
+
     pub fn inclusive_range(s: char, e: char) -> Test {
         Test {
             start: s as u32,
@@ -509,6 +530,13 @@ impl Test {
         }
     }
 
+    pub fn inclusive_byte_range(s: u8, e: u8) -> Test {
+        Test {
+            start: s as u32,
+            end: e as u32 + 1,
+        }
+    }
+
     pub fn exclusive_range(s: char, e: char) -> Test {
         Test {
             start: s as u32,
@@ -553,6 +581,12 @@ impl From<ClassUnicodeRange> for Test {
     }
 }
 
+impl From<ClassBytesRange> for Test {
+    fn from(range: ClassBytesRange) -> Test {
+        Test::inclusive_byte_range(range.start(), range.end())
+    }
+}
+
 impl Debug for Test {
     fn fmt(&self, fmt: &mut Formatter) -> Result<(), FmtError> {
         match (char::from_u32(self.start), char::from_u32(self.end)) {
