@@ -2,18 +2,18 @@
 //!
 //! [recursive ascent]: https://en.wikipedia.org/wiki/Recursive_ascent_parser
 
-use collections::Multimap;
-use grammar::repr::{
+use crate::collections::Multimap;
+use crate::grammar::repr::{
     Grammar, NonterminalString, Production, Symbol, TerminalString, TypeParameter, TypeRepr,
     Visibility, WhereClause,
 };
-use lr1::core::*;
-use lr1::lookahead::Token;
-use lr1::state_graph::StateGraph;
-use rust::RustWrite;
+use crate::lr1::core::*;
+use crate::lr1::lookahead::Token;
+use crate::lr1::state_graph::StateGraph;
+use crate::rust::RustWrite;
+use crate::tls::Tls;
+use crate::util::{Escape, Sep};
 use std::io::{self, Write};
-use tls::Tls;
-use util::{Escape, Sep};
 
 use super::base::CodeGenerator;
 
@@ -180,7 +180,7 @@ impl<'ascent, 'grammar, W: Write>
         rust!(self.out, "#[allow(dead_code)]");
         rust!(
             self.out,
-            "pub enum {}Nonterminal<{}>",
+            "pub(crate) enum {}Nonterminal<{}>",
             self.prefix,
             Sep(", ", &self.custom.nonterminal_type_params)
         );
@@ -376,7 +376,7 @@ impl<'ascent, 'grammar, W: Write>
                     .any(|&(ref t, _)| t.contains(&Token::Terminal(terminal.clone())))
         });
 
-        rust!(self.out, "let {}expected = vec![", self.prefix);
+        rust!(self.out, "let {}expected = alloc::vec![", self.prefix);
         for terminal in successful_terminals {
             rust!(self.out, "r###\"{}\"###.to_string(),", terminal);
         }
@@ -567,11 +567,11 @@ impl<'ascent, 'grammar, W: Write>
             )))
             .with_parameters(fn_args)
             .with_return_type(format!(
-                "Result<(Option<{}>, {}Nonterminal<{}>), {}>",
+                "core::result::Result<(core::option::Option<{}>, {}Nonterminal<{}>), {}>",
                 triple_type,
                 self.prefix,
                 Sep(", ", &self.custom.nonterminal_type_params),
-                parse_error_type
+                parse_error_type,
             ))
             .emit()?;
 
@@ -579,11 +579,11 @@ impl<'ascent, 'grammar, W: Write>
 
         rust!(
             self.out,
-            "let mut {}result: (Option<{}>, {}Nonterminal<{}>);",
+            "let mut {}result: (core::option::Option<{}>, {}Nonterminal<{}>);",
             self.prefix,
             triple_type,
             self.prefix,
-            Sep(", ", &self.custom.nonterminal_type_params)
+            Sep(", ", &self.custom.nonterminal_type_params),
         );
 
         // shift lookahead is necessary; see `starts_with_terminal` above
@@ -627,18 +627,21 @@ impl<'ascent, 'grammar, W: Write>
 
         let mut base_args = vec![format!("{}tokens: &mut {}TOKENS", self.prefix, self.prefix)];
         if !starts_with_terminal {
-            base_args.push(format!("{}lookahead: Option<{}>", self.prefix, triple_type));
+            base_args.push(format!(
+                "{}lookahead: core::option::Option<{}>",
+                self.prefix, triple_type,
+            ));
         }
 
         // "Optional symbols" may or may not be consumed, so take an
         // `&mut Option`
         let optional_args = (0..optional_prefix.len()).map(|i| {
             format!(
-                "{}sym{}: &mut Option<{}>",
+                "{}sym{}: &mut core::option::Option<{}>",
                 self.prefix,
                 i,
                 self.types
-                    .spanned_type(optional_prefix[i].ty(&self.types).clone())
+                    .spanned_type(optional_prefix[i].ty(&self.types).clone()),
             )
         });
 
@@ -843,17 +846,18 @@ impl<'ascent, 'grammar, W: Write>
         };
         let transfer_syms = self.pop_syms(optional, fixed, production_inputs)?;
 
-        // identify the "start" location for this production; this
-        // is typically the start of the first symbol we are
-        // reducing; but in the case of an empty production, it
-        // will be the last symbol pushed, or at worst `default`.
-        if let Some(first_sym) = transfer_syms.first() {
+        // identify the "start" and "end" location for this production; this
+        // is typically the start of the first symbol and end of the last symbol we are
+        // reducing; but in the case of an empty production, it will come from the
+        // lookahead or the end of the last symbol pushed
+        if let (Some(first_sym), Some(last_sym)) = (transfer_syms.first(), transfer_syms.last()) {
             rust!(
                 self.out,
                 "let {}start = {}.0.clone();",
                 self.prefix,
                 first_sym
             );
+            rust!(self.out, "let {}end = {}.2.clone();", self.prefix, last_sym);
         } else if stack_suffix.len() > 0 {
             // we pop no symbols, so grab from the top of the stack
             // (unless we are in the start state)
@@ -861,46 +865,29 @@ impl<'ascent, 'grammar, W: Write>
             if !stack_suffix.fixed().is_empty() {
                 rust!(
                     self.out,
-                    "let {}start = {}sym{}.2.clone();",
-                    self.prefix,
-                    self.prefix,
-                    top
+                    "let {p}start = {p}lookahead.as_ref().map(|o| o.0.clone()).unwrap_or_else(|| {p}sym{top}.2.clone());",
+                    p = self.prefix,
+                    top = top
                 );
             } else {
                 // top of stack is optional; should not have been popped yet tho
                 rust!(
                     self.out,
-                    "let {}start = {}sym{}.as_ref().unwrap().2.clone();",
-                    self.prefix,
-                    self.prefix,
-                    top
+                    "let {p}start = {p}lookahead.as_ref().map(|o| o.0.clone()).unwrap_or_else(|| {p}sym{top}.as_ref().unwrap().2.clone());",
+                    p = self.prefix,
+                    top = top
                 );
             }
+            rust!(self.out, "let {p}end = {p}start;", p = self.prefix);
         } else {
             // this only occurs in the start state
             rust!(
                 self.out,
-                "let {}start: {} = ::std::default::Default::default();",
+                "let {}start: {} = core::default::Default::default();",
                 self.prefix,
-                loc_type
-            );
-        }
-
-        // identify the "end" location for this production;
-        // this is typically the end of the last symbol we are reducing,
-        // but in the case of an empty production it will come from the
-        // lookahead
-        if let Some(last_sym) = transfer_syms.last() {
-            rust!(self.out, "let {}end = {}.2.clone();", self.prefix, last_sym);
-        } else {
-            rust!(
-                self.out,
-                "let {}end = {}lookahead.as_ref().map(|o| o.0.clone()).unwrap_or_else(|| \
-                 {}start.clone());",
-                self.prefix,
-                self.prefix,
-                self.prefix
+                loc_type,
             );
+            rust!(self.out, "let {p}end = {p}start;", p = self.prefix);
         }
 
         let transfered_syms = transfer_syms.len();
