@@ -1,16 +1,16 @@
 //! Error reporting. For now very stupid and simplistic.
 
-use collections::{set, Set};
-use grammar::repr::*;
+use crate::collections::{set, Set};
+use crate::grammar::repr::*;
 use itertools::Itertools;
-use lr1::core::*;
-use lr1::example::{Example, ExampleStyles, ExampleSymbol};
-use lr1::first::FirstSets;
-use lr1::lookahead::{Token, TokenSet};
-use lr1::trace::Tracer;
-use message::builder::{BodyCharacter, Builder, Character, MessageBuilder};
-use message::Message;
-use tls::Tls;
+use crate::lr1::core::*;
+use crate::lr1::example::{Example, ExampleStyles, ExampleSymbol};
+use crate::lr1::first::FirstSets;
+use crate::lr1::lookahead::{Token, TokenSet};
+use crate::lr1::trace::Tracer;
+use crate::message::builder::{BodyCharacter, Builder, Character, MessageBuilder};
+use crate::message::Message;
+use crate::tls::Tls;
 
 #[cfg(test)]
 mod test;
