@@ -1,8 +1,8 @@
-use collections::{set, Set};
-use grammar::repr::*;
-use lr1::core::*;
-use lr1::first::FirstSets;
-use lr1::state_graph::StateGraph;
+use crate::collections::{set, Set};
+use crate::grammar::repr::*;
+use crate::lr1::core::*;
+use crate::lr1::first::FirstSets;
+use crate::lr1::state_graph::StateGraph;
 
 mod reduce;
 mod shift;
