@@ -46,6 +46,7 @@ pub enum Tok<'input> {
     If,
     Mut,
     Pub,
+    In,
     Type,
     Where,
     For,
@@ -127,6 +128,7 @@ const KEYWORDS: &[(&str, Tok<'static>)] = &[
     ("if", If),
     ("mut", Mut),
     ("pub", Pub),
+    ("in", In),
     ("type", Type),
     ("where", Where),
     ("for", For),
